intersect.roc <- function(roc.obj) {

## find max sensitivity and specificity for each marker and print results

   if(!inherits(roc.obj,"roc"))
        stop("Not legitimate roc object")
   if(!is.null(cl <- roc.obj$call)) {
	     cat("Call:\n")
	     dput(cl)
	     cat("\n")
	   }

   markers <- roc.obj$x
   nmark <- ncol(markers)
   status <- roc.obj$y
   marker.names <- attr(roc.obj$terms,"term.labels")
   sens <- roc.obj$sensitivity
   spec <- roc.obj$specificity

   min.match <- apply(abs(sens-spec),2,min)
   locate.min.match <- matrix(0,nrow=nmark,ncol=1)

   minD.match <- apply(sqrt((1-sens)^2 + (1-spec)^2),2,min)
   locate.minD.match <- matrix(0,nrow=nmark,ncol=1)

   for (i in 1:nmark)
      {locate.min.match[i] <- match(min.match[i],(abs(sens-spec))[,i])
       locate.minD.match[i] <- match(minD.match[i],(sqrt((1-sens)^2 + (1-spec)^2))[,i])
     }

   if(nmark==1) {
       x.min.match <- markers[locate.min.match,1:nmark]
       sens.min.match <- sens[locate.min.match,1:nmark]
       spec.min.match <- spec[locate.min.match,1:nmark]

       x.minD.match <- markers[locate.minD.match,1:nmark]
       sens.minD.match <- sens[locate.minD.match,1:nmark]
       spec.minD.match <- spec[locate.minD.match,1:nmark]
        }

   if(nmark>1) {
       x.min.match <- diag(markers[locate.min.match,1:nmark])
       sens.min.match <- diag(sens[locate.min.match,1:nmark])
       spec.min.match <- diag(spec[locate.min.match,1:nmark])

       x.minD.match <- diag(markers[locate.minD.match,1:nmark])
       sens.minD.match <- diag(sens[locate.minD.match,1:nmark])
       spec.minD.match <- diag(spec[locate.minD.match,1:nmark])
     }

   ans.min <- t(rbind(x.min.match,sens.min.match,spec.min.match))
   dimnames(ans.min) <- list(marker.names,
			 c('Marker value','Sensitivity','Specificity'))

   ans.minD <- t(rbind(x.minD.match,sens.minD.match,spec.minD.match))
   dimnames(ans.minD) <- list(marker.names,
			 c('Marker value','Sensitivity','Specificity'))
   cat("The 'best' cutoffs maximizing sensitivity and specificity:\n\n")
   cat("When minimizing abs(sensitivity-specificity)\n\n")
   print(ans.min)
   cat("\n")

   cat("When minimizing sqrt((1-sensitivity)^2 + (1-specificity)^2)\n\n")
   print(ans.minD)
   cat("\n\n")
   invisible()

 }

